;**********************************************************************
;                                                                     *
;    NOM:       portail627v4.asm                                      *
;    Date:      16/06/05                                              *
;    Version:   4                                                     *
;    Circuit:	Commande de portail 2 battants                        *
;    Auteur:  	Frdric BUISSON                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F627.inc                                      *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:   Commande d'un portail a 2 battants differents avec      *
;             retard de 2s pour un battant (le petit  l'ouverture,   *
;             le grand  la fermeture). Inversion de sens autoris    *
;             en cours de mouvement.            					  *
;			  Teste si le portail n'est plus sur les fin de course    *  
;             et le replace toute les secondes.                       *
;**********************************************************************

;*********************************************************************
;                              EXPLICATIONS                          *
;*********************************************************************


; Petit schma expliquant la diffrence entre les diffrents battants

; Petit battant          Grand Battant

; --------------------------- |------------------------------
;                           --|
; On voit ici que le grand battant englobe le petit
; Les battants ne doivent pas etre dplacs en meme temps
; donc j'ai ajout une temporisation de 2 secondes entre les 2 battants

;*********************************************************************
;                                 DEBUT                              *
;*********************************************************************

	LIST      p=16F627            ; Dfinition de processeur
	#include <p16F627.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_OFF & _HS_OSC & _LVP_OFF & _DATA_CP_OFF & _BODEN_OFF & _MCLRE_OFF

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Voici les valeurs et leurs dfinitions :
;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service
;	_WDT_ON		Watch-dog en service
;	_WDT_OFF	Watch-dog hors service
;	_LP_OSC		Oscillateur quartz basse vitesse
;	_XT_OSC		Oscillateur quartz moyenne vitesse
;	_HS_OSC		Oscillateur quartz grande vitesse
;	_RC_OSC		Oscillateur  rseau RC
;	_LVP_OFF	Pas de programmation basse tension
;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	H'CF'	; Valeur registre option
                                ; Rsistance pull-up OFF
                                ; Interruption front montant RB0
                                ; Prscaler timer0  256 

INTERMASK	EQU	H'C0' 	; Masque d'interruption servira pour INTCON
RPIE1	EQU	H'03' ; Interruption sur timer1 et 2

RTRISB 	EQU H'FF' ; PORTB en entre
RTRISA 	EQU H'F0' ; PORTA en sortie
RCMCON	EQU	H'07' ; PORTA en numrique

RT1H	EQU	H'0B' ; Octet de poids fort du timer1
RT1L	EQU	H'E0' ; Octet de poids faible du timer1
RT1CON	EQU H'30' ; Synchronisation sur Tosc*8 500ms

RT2		EQU H'06' ; Octet du timer 4ms
RT2CON	EQU H'03' ; Prdivision par 16 timer off

RCMPT1	EQU H'04' ; Initialis pour 4	
RCMPT2	EQU H'FA' ; Initialis pour 250

RMOUV	EQU H'02' ; portail 01->Ouverture 10-> Fermeture

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************


; entres
; ---------
#DEFINE CGF	PORTB,1		; Contact du grand battant ferm
#DEFINE CGO	PORTB,2		; Contact du grand battant ouvert
#DEFINE CPF	PORTB,3		; Contact du petit battant ferm
#DEFINE CPO	PORTB,4		; Contact du petit battant ouvert
#DEFINE MARCHE	PORTB,0		; Demande d'ouverture/fermeture

; sorties
; ---------
#DEFINE OG	PORTA,0		; Ouverture du grand battant
#DEFINE FG	PORTA,1		; Fermeture du grand battant
#DEFINE OP	PORTA,2		; Ouverture du petit battant
#DEFINE FP	PORTA,3		; Fermeture du petit battant


;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

;exemples
;---------
	CBLOCK 0x020   		; dbut de la zone variables
	w_temp 			:	1	; Zone de 1 byte
	status_temp 	: 	1	; zone de 1 byte
	cmpt1 			: 	1 	; compteur1 pour timer1
	cmpt2 			:	1 	; compteur2 pour timer2
	mouv 			: 	1 	;sert  donner le sens des battants
    ENDC				; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init	; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------

	org 0x004			; adresse d'interruption
	
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	bcf 	STATUS,RP0
	movwf	status_temp	; sauver status swapp		
	bcf		INTCON,GIE	; Dsactive les interruptions
		
			;Test du type d'interruption	
			;---------------------------

	btfsc	PIR1,TMR1IF	
	goto 	int_t1
	goto 	int_t2

			;Interruption du timer1	
			;----------------------

int_t1	
	movlw	RT1H		; Init octet de poids fort timer 1
	movwf	TMR1H
	movlw	RT1L		; Init octet de poids faible timer 1
	movwf	TMR1L
	decf	cmpt1,f
	bcf		PIR1,TMR1IF	; effacer flag interrupt timer 1
	goto 	restorereg

			;Interruption du timer2	
			;----------------------

int_t2	
	movlw	RT2			; Init du timer 2
	movwf	TMR2	
	decf	cmpt2,f
	bcf		PIR1,TMR2IF	; effacer flag interrupt timer 2
	goto 	restorereg

			;restauration des registres	
			;--------------------------

restorereg
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       	; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
						; W restaur sans modifier status
	bsf 	INTCON,GIE	; Ractive les interruptions
	retfie  			; Fin d'interruption

;*********************************************************************
;                        SOUS-PROGRAMME                              *
;*********************************************************************

			; Sous-prog ouverture	
			;---------------------

ouverture
	clrf  	PORTA		; Annule toutes les actions
	btfss	CGO			; Si grand battant non ouvert
	bsf		OG			; Ouvrir le grand battant
	movlw	RT1H		; Init octet de poids fort timer 1
	movwf	TMR1H
	movlw	RT1L		; Init octet de poids faible timer 1
	movwf	TMR1L
	movlw 	RCMPT1		; Initialise le compteur 2s
	movwf	cmpt1
	bsf		T1CON,TMR1ON; Active la tempo	

tempo1
	btfsc	CGO			; Si grand battant ouvert
	bcf		OG			; Grand battant immobilis
	movfw	cmpt1		
	btfss	STATUS,Z	; Test si la valeur est 0
	goto 	tempo1			; non retour au test
	bcf		T1CON,TMR1ON; Desactive la tempo
	btfsc	CPO			; Si petit battant non ouvert
	bsf		OP			; Ouvrir le petit battant

ouverture_suite	
	btfsc	MARCHE		; Prvu pour changer de sens
	goto	direction	; en fonctionnement
	btfss	CGO			; Si grand battant non ouvert
	bsf		OG			; Ouvrir le grand battant
	btfss	CPO			; Si petit battant non ouvert
	bsf		OP			; Ouvrir le petit battant
	btfsc	CGO			; Si grand battant ouvert
	bcf		OG			; Grand battant immobilis
	btfsc	CPO			; Si petit battant ouvert
	bcf		OP			; Petit battant immobilis
; Tests de fin d'ouverture
	btfss	CGO
	goto	ouverture_suite	
	btfss	CPO			
	goto 	ouverture_suite
	goto 	stop		; fin du programme

			; Sous-prog fermeture	
			;---------------------

fermeture
	clrf  	PORTA		; Annule toutes les actions
	btfss	CPF			; Si petit battant non ouvert
	bsf		FP			; Ouvrir le petit battant
	movlw	RT1H		; Init octet de poids fort timer 1
	movwf	TMR1H
	movlw	RT1L		; Init octet de poids faible timer 1
	movwf	TMR1L
	movlw 	RCMPT1		; Initialise le compteur 2s
	movwf	cmpt1	
	bsf		T1CON,TMR1ON; Active la tempo

tempo2
	btfsc	CPF			; Si grand battant ouvert
	bcf		FP			; Grand battant immobilis
	movfw	cmpt1		
	btfss	STATUS,Z	; Test si la valeur est 0
	goto 	tempo2			; non retour au test
	bcf		T1CON,TMR1ON; Desactive la tempo
	btfsc	CGF			; Si petit battant non ouvert
	bsf		FG			; Ouvrir le petit battant

fermeture_suite
	btfsc	MARCHE		; Prvu pour changer de sens
	goto	direction	; en fonctionnement
	btfss	CPF			; Si petit battant non ferme
	bsf		FP			; Fermer le petit battant
	btfss	CGF			; Si petit battant non ferme
	bsf		FG			; Fermer le petit battant
	btfsc	CGF			; Si grand battant ferm
	bcf		FG			; Grand battant immobilis
	btfsc	CPF			; Si petit battant ferm
	bcf		FP			; Petit battant immobilis
; Tests de fin de fermeture
	btfss	CPF
	goto	fermeture_suite	
	btfss	CGF			
	goto 	fermeture_suite
	goto 	stop		; fin du programme

			; Sous-prog verification	
			;-----------------------

verif
	bcf		T2CON,TMR2ON; Timer 2 off
	btfsc 	mouv,0		; Test si on doit etre ouvert
	goto  	ouverture_suite
	btfsc 	mouv,1		; Test si on doit etre ferm
	goto	fermeture_suite
	goto 	init		; normalement pas possible											
	

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrwdt				; kick le watchdog
	bcf		STATUS,RP0	; slectionner banque 0
	bcf		STATUS,RP1	; slectionner banque 0	
	movlw	RCMCON
	movwf	CMCON		; PortA en numrique
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
	bsf		STATUS,RP0	; slectionner banque 1
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option
	bcf		STATUS,RP0	; slectionner banque 0

			; Effacer RAM
			; ------------
	movlw	0x20		; initialisation pointeur
	movwf	FSR			; pointeur d'adressage indirec
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (>=50)
	goto	init1		; non, boucler

			;Configuration des ports
			;-----------------------
	bsf 	STATUS,RP0 	; on passe en banque 1
	movlw	RPIE1		; initialiser registre PIE1
	movwf	PIE1		; charger config du PIE1
	clrf	VRCON		; initialiser registre VRCON
	movlw	RTRISA		; charger config du PORTA
	movwf	TRISA		; initialiser registre TRISA
	movlw	RTRISB		; charger config du PORTB
	movwf	TRISB		; initialiser registre TRISB
	bcf 	STATUS,RP0 	; on passe en banque 0
	movlw 	RMOUV		; Initialise le sens de rotation sur stop
	movwf	mouv		
	movlw	RT1CON
	movwf	T1CON
	movlw	RT2			; Init du timer 2
	movwf	TMR2
	movlw	RT2CON
	movwf	T2CON
	movlw 	RCMPT2		; Initialise le compteur2
	movwf	cmpt2
	bsf		T2CON,TMR2ON; Timer 2 on
	movlw	INTERMASK
	movwf	INTCON		; Activation des interruptions		

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start
	movfw 	cmpt2		; Test si on a attendu 1s	
	btfsc	STATUS,Z	
	goto 	verif		; Lance la routine de verification
	btfss	MARCHE		; Test si un ordre a t donn 
	goto 	start		; non retour au test
	bcf		T2CON,TMR2ON; Timer 2 off
direction
	clrf	PORTA		; Sert seulement lors d'un changement de direction
	comf 	mouv,f		; On complemente mouv
	movlw 	0x03		; Chargement du masque
	andwf 	mouv,f		; Masque les bits  partir du 2e
	btfsc	MARCHE		; Test si le BP est relach
	goto	$-1			; Attend tant que le BP n'est pas relach
	btfsc 	mouv,0		; Test si un ordre d'ouverture
	goto  	ouverture
	btfsc 	mouv,1		; Test si un ordre de fermeture
	goto 	fermeture	
stop
	clrf 	PORTA		; Aucune action 
	movlw 	RCMPT2		; Initialise le sens de rotation sur stop
	movwf	cmpt2			
	movlw	RT2			; Init du timer 2
	movwf	TMR2		
	bsf		T2CON,TMR2ON; Timer 2 on
	goto 	start

	END 				; directive fin de programme
